@@ -1,7 +1,7 @@
 // Copyright (c) NetXS Group.
 // Licensed under the MIT license.
 
-#define DESKTOPIO_VER "v0.8.0"
+#define DESKTOPIO_VER "v0.9.0"
 #define DESKTOPIO_MYNAME "vtm " DESKTOPIO_VER
 #define DESKTOPIO_PREFIX "desktopio_"
 #define DESKTOPIO_MYPATH "vtm"
@@ -27,10 +27,11 @@ int main(int argc, char* argv[])
 {
     auto vtmode = os::vt_mode();
     auto syslog = os::ipc::logger(vtmode);
-    auto banner = [&]() { log(DESKTOPIO_MYNAME); };
+    auto banner = [&]{ log(DESKTOPIO_MYNAME); };
     auto whoami = type::client;
     auto params = text{};
     auto maxfps = si32{ 60 };
+
     {
         auto daemon = faux;
         auto getopt = os::args{ argc, argv };
@@ -150,7 +151,7 @@ int main(int argc, char* argv[])
             auto usernm = os::get_env("USER");
             auto hostip = os::get_env("SSH_CLIENT");
             auto prefix = utf::concat(DESKTOPIO_PREFIX, userid);
-            auto client = os::ipc::open<os::client>(prefix, 10s, [&]()
+            auto client = os::ipc::open<os::client>(prefix, 10s, [&]
                         {
                             log("main: new desktopio environment for user ", userid);
                             auto binary = os::current_module_file();
